# Copyright (C) 2000, 2001  SWsoft
# All rights reserved.
#

# Object-Oriented wrapper into HSPC::WebPage
# This class implements Abstract dashboard page class

package HSPC::AbstractDashboard;

use strict;

use HSPC::Localization;
use HSPC::AbstractPage;
use Data::Dumper;

use HSPC::WebSystemLib;
use SWsoft::protect;

use vars qw( @ISA );

@ISA = qw( HSPC::AbstractPage );

use constant _DASH_	=> 101;

sub handler {
	my $self = shift;

	$self->parse_request;

	$self->show_dashboard;

	$self->footer;
}

sub title {
	my $self = shift;
	
	return HSPC::Localization::string($self->{_descriptior_}->{app_title_id});
}

sub devscreen_id {
	my $self = shift;
	
	return $self->{_descriptior_}->{app_devscreen_id};
}


sub show_dashboard {
	my $self = shift;

	$self->mode(_DASH_);

	$self->header;

	my %parm = ();

	# summary_title
	if ( defined $self->{_descriptior_}->{summary_title_id} ) {
		$parm{summary_title} = HSPC::Localization::string($self->{_descriptior_}->{summary_title_id});
	}
	elsif ( defined $self->{_descriptior_}->{summary_title} ) {
		$parm{summary_title} = $self->{_descriptior_}->{summary_title};
	}
	else {
		$parm{summary_title} = HSPC::Localization::string('summary');
	}

	# action_title
	if ( defined $self->{_descriptior_}->{action_title_id} ) {
		$parm{action_title} = HSPC::Localization::string($self->{_descriptior_}->{action_title_id});
	}
	elsif ( defined $self->{_descriptior_}->{action_title} ) {
		$parm{action_title} = $self->{_descriptior_}->{action_title};
	}
	else {
		$parm{action_title} = HSPC::Localization::string('go_to');
	}
	$parm{summary_url} = $self->{_descriptior_}->{summary_url} if defined $self->{_descriptior_}->{summary_url};

	# base_url
	$parm{url} = defined $self->{_descriptior_}->{base_url} ? $self->{_descriptior_}->{base_url} : '';

	# localization
	my $summary = $self->summary;
	my $actions = $self->actions;
	foreach ( $summary, $actions ) {
		foreach ( @$_ ) {
			next unless $_->{title_id};
			$_->{title} = HSPC::Localization::string($_->{title_id});
		}
	}

	$self->{_page_}->dashboard (
		%parm,
		summary	=> $summary,
		action	=> $actions
	);
}

sub summary {
	return [];
}

sub actions {
	return [];
}

1;

__END__
