# Paging handling functions
# Author: olsh@

use strict;
package HSPC::Pager;
require 5.000;
require Exporter;
use HSPC::Config;
use Text::Template;
use HSPC::Localization;
use SWsoft::protect;	# Mark this module for encryption upon build

#use Carp;
#use CGI qw(:standard);
#use CGI::Cookie;
#use Time::gmtime;
#use DBI ();
#use Apache::Session::MySQL;
#use Apache;

use vars qw(
	@ISA @EXPORT_OK @EXPORT %EXPORT_TAGS %_tmpl
);

@ISA = qw(Exporter);
@EXPORT = qw(Show_Pager Show_Titles);
@EXPORT_OK = qw(Show_Pager Show_Titles);
%EXPORT_TAGS = ();

# Pager display function, shows navigation bar
# Prototype:
#	Show_Pager(session => $session, total => $total_count, name => $item_name);
# Parameters:
#	Required:
#		session		- Reference to %session hash
#		total		- Total number of items in list
#	Optional:
#		name		- Items Name (by default ='Items')
#		base_url	- Base URL
#		per_page	- per-page possible values
#		template	- Name of template to be applied
#		list_id		- Unique list alias to distinguish this list from others (script name used by default)
# Return value:
#	($top_out, $bottom_out, $from, $pagesize)
#	$top_out	- HTML code of TOP navigation bar
#	$bottom_out	- HTML code of BOTTOM navigation bar
#	$from		- start position in the list to display records
#	$pagesize	- number of records to be displayed on single page
#
sub Show_Pager { # display pager line
	my %arg = (
		total		=>	0,
		name		=>	'Items',
		base_url	=>	$ENV{REQUEST_URI},
		per_page	=>	[ 20, 50, 100],			# per-page possible values
		max_links	=>	10,						# maximum number of allowed page-links
		list_id		=>	$ENV{SCRIPT_NAME},
		@_ );
	$arg{base_url} =~ s/(\?|\&)pg[^\&]*(\&|$)/$1/g;
	if ($arg{base_url} !~ /\?/) {
		$arg{base_url} .= '?';
	} elsif ($arg{base_url} !~ /\?$/ && $arg{base_url} !~ /\&$/) {
		$arg{base_url} .= '&';
	}

	unless (exists $arg{session}) {
		warn ("Incorrect reference to session passed as argument!\n");
		$arg{session}={}; # just to shut up pager in bad situations
	}
	
	my ($total_count, $start, $to, $pagesize);

	# determine total number of rows and a start position
	$total_count = exists $arg{session}->{'pg_'.$arg{list_id}.'_count'} ? $arg{session}->{'pg_'.$arg{list_id}.'_count'} : 0;
	$start = exists $arg{session}->{'pg_'.$arg{list_id}.'_start'} ? $arg{session}->{'pg_'.$arg{list_id}.'_start'} : 1;
	if($arg{total}!=$total_count) { $total_count = $arg{total}; $start=1; }
	$start = $1 if($ENV{REQUEST_URI} =~ /[\?\&]pgs\=(\d+)/);
	$start = 1 unless $start>0;

	# determine pagesize
	$pagesize = exists $arg{session}->{'pg_'.$arg{list_id}.'_ps'} ? $arg{session}->{'pg_'.$arg{list_id}.'_ps'} : $arg{per_page}->[0];
	$pagesize = $1 if($ENV{REQUEST_URI} =~ /[\?\&]pgps\=(\d+)/);
	$pagesize = $arg{per_page}->[0] if !($pagesize>0);

	# verify start position and calculate end position
	$start = $total_count if ($start>$total_count);
	$start = int($start/$pagesize)*$pagesize + 1;
	$to = ($total_count > $start+$pagesize-1) ? $start+$pagesize-1 : $total_count;

	# save values in session
	$arg{session}->{'pg_'.$arg{list_id}.'_count'}=$total_count;
	$arg{session}->{'pg_'.$arg{list_id}.'_start'}=$start;
	$arg{session}->{'pg_'.$arg{list_id}.'_ps'}=$pagesize;

	# calculate page numbers to show
	my ($p_from, $p_to);
	my $p_cur = int(($start-1)/$pagesize)+1;
	my $p_last = int(($total_count-1)/$pagesize)+1;
	if($p_cur > $arg{max_links}/2 && $p_last > $arg{max_links}) {
		if($p_last > $p_cur + $arg{max_links}/2 - 1) {
			$p_from = $p_cur-$arg{max_links}/2;
			$p_to = $p_cur+$arg{max_links}/2;
		} else {
			$p_from = $p_last - $arg{max_links};
			$p_to = $p_last;
		}
	} else {
		$p_from = 1;
		$p_to = ($p_last > $arg{max_links}) ? $arg{max_links} : $p_last;
	}

	my %vars = ( %arg,
			start => $start, to => $to,
			total_count => $total_count, pagesize => $pagesize,
			p_from => $p_from, p_to => $p_to, p_cur => $p_cur
		);
	delete $vars{session};
	$vars{p_next} = $p_cur+1 if $p_last > $p_cur;
	$vars{p_prev} = 1 if $p_cur > 1;
	#%items% per page: %index%
	$vars{per_page_str} = &string('item_per_page');
	$vars{per_page_str} =~ s/\%items\%/$vars{name}/;
	#Page
	$vars{page_str} = &string('page');
	#%items% %from% - %to% of %count%
	$vars{item_from_to_str} = &string('item_from_to');
	$vars{item_from_to_str} =~ s/\%items\%/$vars{name}/;
	$vars{item_from_to_str} =~ s/\%from\%/$vars{start}/;
	$vars{item_from_to_str} =~ s/\%to\%/$vars{to}/;
	$vars{item_from_to_str} =~ s/\%count\%/$vars{total_count}/;
	
	# load template if not loaded yet
	$arg{template} = exists $arg{template} ? $CONFIG{PATHTOTEMPLATES}.$arg{template} : $CONFIG{OFFICEPATH}.'template/pg_navbar.tmpl';
	unless (exists $_tmpl{$arg{template}}) {
		$_tmpl{$arg{template}} = new Text::Template (SOURCE => $arg{template})
			or die "Couldn't construct template: $Text::Template::ERROR";
	}

	$vars{position}='top';
	my $top_out = $_tmpl{$arg{template}}->fill_in(HASH => \%vars);
	$vars{position}='bottom';
	my $bottom_out = $_tmpl{$arg{template}}->fill_in(HASH => \%vars);

	return ($top_out, $bottom_out, ($p_cur-1)*$pagesize+1, $pagesize);
}


# Pager display function, shows navigation bar
# Prototype:
#	Show_Titles(session => $session, columns => $hashref_array);
# Parameters:
#	Required:
#		session		- Reference to %session hash
#		columns		- array of references to hash with keys: name, alias, width, type [, defSort, custom_text]
#	Optional:
#		base_url	- Base URL
#		template	- name of template to be applied
#		list_id		- Unique list alias to distinguish this list from others (script name used by default)
# Return value:
# 	($out, $sort_alias, $sort_direction)
#	$out - HTML code of list header with column titles
#	$sort_alias - alias of the sorted column
#	$sort_direction - ascending (+1) or descending (-1)
#
sub Show_Titles {
	my %arg = (
		columns		=>	[],
		base_url	=>	$ENV{REQUEST_URI},
		list_id		=>	$ENV{SCRIPT_NAME},
		@_ );
	$arg{base_url} =~ s/(\?|\&)pg[^\&]*(\&|$)/$1/g;
	if ($arg{base_url} !~ /\?/) {
		$arg{base_url} .= '?';
	} elsif ($arg{base_url} !~ /\?$/ && $arg{base_url} !~ /\&$/) {
		$arg{base_url} .= '&';
	}

	unless (exists $arg{session}) {
		warn ("Incorrect reference to session passed as argument!\n");
		$arg{session}={}; # just to shut up pager in bad situations
	}

	my ($set_srt_col, $srt_col, $srt_dir)=();
	if(exists $arg{session}->{'pg_'.$arg{list_id}.'_sort_col'} && $arg{session}->{'pg_'.$arg{list_id}.'_sort_col'}) {
		$set_srt_col = $arg{session}->{'pg_'.$arg{list_id}.'_sort_col'};
		$srt_dir = $arg{session}->{'pg_'.$arg{list_id}.'_sort_dir'};
	}
	($set_srt_col, $srt_dir) = ($1, $2) if($ENV{REQUEST_URI} =~ /[\?\&]pgsrt\_([^\=]*)\=([\-\+]*\d+)/);

	foreach my $column (@{$arg{columns}}) {
		if(defined($set_srt_col) && exists $column->{alias} && $column->{alias} eq $set_srt_col) {
			$srt_col = $set_srt_col;
		} elsif(!defined($set_srt_col) and exists $column->{def_sort}) {
			$srt_col = $column->{alias};
			$srt_dir = $column->{def_sort};
		}
	}
	$srt_dir = ($srt_dir==-1) ? -1 : 1;
	if(defined($srt_col) && $srt_col) {
		$arg{session}->{'pg_'.$arg{list_id}.'_sort_col'} = $srt_col;
		$arg{session}->{'pg_'.$arg{list_id}.'_sort_dir'} = $srt_dir;
	}
 

	$arg{template} = exists $arg{template} ? $CONFIG{PATHTOTEMPLATES}.$arg{template} : $CONFIG{OFFICEPATH}.'template/pg_titles.tmpl';
	unless (exists $_tmpl{$arg{template}}) {
		$_tmpl{$arg{template}} = new Text::Template (SOURCE => $arg{template})
			or die "Couldn't construct template: $Text::Template::ERROR";
	}

	my %vars = (%arg, srt_col => $srt_col, srt_dir => $srt_dir);
	delete $vars{session};
	my $out = $_tmpl{$arg{template}}->fill_in(HASH => \%vars);
	return ($out, $srt_col, $srt_dir);
}

1;
