package HSPC::Session;

use strict;

use SWsoft::protect;	# Mark this module for encryption upon build

use HSPC::WebDB;
use HSPC::Config;
use HSPC::Localization;

use Apache::Session::MySQL;

# > COOKIE_IN
# > COOKIE_OUT
# > DBH

sub checkSession{
	my ($COOKIE_IN, $COOKIE_OUT, $DBH, $sid) = @_;
	$sid ||= $$COOKIE_IN{sid}->value
		if ref $COOKIE_IN eq 'HASH' && exists $$COOKIE_IN{sid};
	my %SESSION;
		
	
	if(defined $DBH){
		$DBH->do('SET NAMES utf8');
		eval {
			tie %SESSION, 'Apache::Session::MySQL', $sid, {
				Handle     => $DBH,
				LockHandle => $DBH
			};
		};
		if ($@) {
			# Create a new session oblect
			eval {
				tie %SESSION, 'Apache::Session::MySQL', undef, {
					Handle     => $DBH,
					LockHandle => $DBH
				};
			};
			if ($@) {
				warn 'ERROR: '.$@;
				die;
			};
		}		
	}else{
		my $dsn =
			'DBI:mysql:' .  $CONFIG{SESSION_DB_NAME} .
			';host=' . $CONFIG{SESSION_DB_HOST};
		my $data  = {
			DataSource	=> $dsn,
			UserName	=> $CONFIG{'SESSION_DB_USER'},
			Password	=> $CONFIG{'SESSION_DB_PASS'},
			LockDataSource	=> $dsn,
			LockUserName	=> $CONFIG{'SESSION_DB_USER'},
			LockPassword	=> $CONFIG{'SESSION_DB_PASS'},
		};
		my $class = 'Apache::Session::MySQL';
		
		eval { tie %SESSION, $class, $sid, $data };
		## Create a new session oblect if SID is not found
		tie %SESSION, $class, undef, $data
			if $@;
	}
	
	$$COOKIE_OUT{sid} = new CGI::Cookie(
		-name    => 'sid',
		-value   => $SESSION{_session_id},
		-path    => '/'
	)	unless ($sid eq $SESSION{_session_id});

	## For checking cookies accessibility
	$$COOKIE_OUT{has_cookies} = new CGI::Cookie(
		-name    => 'has_cookies',
		-value   => 1,
		-path    => '/'
	);

	## For error localization accessibility
	$$COOKIE_OUT{lng} = new CGI::Cookie(
		-name    => 'lng',
		-value   => $SESSION{LANG} || $HSPC::Config::LOCALE{local_lang},
		-path    => '/'
	);

	## update session timestamp (if requested in connection to user)
	$SESSION{_session_timestamp} = time if defined $COOKIE_IN;

	return \%SESSION;
}

## cleanup of expired sessions (obeys SESSION_TIMEOUT parameter from hspc.conf)
sub clean_expired {
	my $db = HSPC::WebDB->new(dbname => 'SESSION');
	foreach (@{$db->select_rows('SELECT `id` FROM `sessions`')}) {
		my $session = checkSession(undef, undef, undef, $_->[0]);
		$db->select_run("DELETE FROM `sessions` WHERE `id`='$_->[0]'")
			if $session->{_session_timestamp}
				&& $session->{_session_timestamp} + ($CONFIG{SESSION_TIMEOUT} || 24) * 3600 < time;
	}
}

1;
