# -*- perl -*-
# HSPC::WebGrid
#
# Old class for multicolumn form creating.
# All WebGrid functionality was moved to WebPage
#
# Author: Bazarov Max
# (c) 2001 SWsoft, All rights reserved

use strict;
package HSPC::WebGrid;

use HSPC::Localization qw(:DEFAULT string string_lang);  

use HSPC::Logger qw(sw_die);
use HSPC::WebSystemLib;
use SWsoft::protect;	# Mark this module for encryption upon build


# >= page #REQ_Y#
# >= cols #REQ_Y#
sub new{
	my $self = {};
	bless ($self,shift);
	my %h=(@_);
				
	$self->{page} = $h{page};		
	
	die unless $self->{page} && $h{cols};
	
	$self->_init(cols=>$h{cols});
		
	return $self;
}

# >= cols #REQ_Y#
sub reinit{
	my $self = shift;
	my %h=(@_);
	
	die 'ERROR: you must call complete() before reinit()' if defined $self->{rows};
	die 'ERROR: undefined cols=> argument for init()' unless defined $h{cols};
	$self->_init(cols=>$h{cols});
}

# >= cols #REQ_Y#
sub _init{
	my $self = shift;
	my %h=(@_);
	
	$self->{cols} = $h{cols};
	
	$self->{rows}=undef;
	$self->reset_row();
}

sub reset_row{
	my $self = shift;
	
	$self->{col} = 0;
	$self->{row} = undef;
}

sub next_row{
	my $self = shift;
	
	# FILL FREE CELLS #
	my $cols = $self->{page}->_row_size();
	while($cols<$self->{cols}){
		$cols++;
		$self->{page}->cell_skip();
	}
	
	$self->{page}->row_close();	
}

# => align, default=center
# => value
# => is_aster
sub cell_text{
	my $self = shift;
	
	$self->{page}->cell_text(@_);
}

# => view_name # REQ_Y #
# => align, default=center
# => value
# => enable_views
# => disable_views
sub cell_check{
	my $self = shift;
	
	$self->{page}->cell_check(@_);
}

# => view_name # REQ_Y #
# => align, default=center
# => value
# => disabled 
# => is_checked
sub cell_radio{
	my $self = shift;
	
	$self->{page}->cell_radio(@_);		
}


sub cell_input{
	my $self = shift;
	$self->{page}->cell_input(@_);
}

# => check_title|check_title_id # REQ_Y #
# => view_name   # REQ_Y #
# => align, default=center
# => read_only
# => size
sub cell_file{
	my $self = shift;
	
	$self->{page}->cell_file(@_);
}


# => check_title|check_title_id # REQ_Y #
# => view_name # REQ_Y #
# => options   # REQ_Y # ref to [[value,res_id],[value1,res_id1]] 
# => options_plain
# => align, default=center
# => value
# => read_only
# => design_width
# => javascript
sub cell_combo{
	my $self = shift;
	
	$self->{page}->cell_combo(@_);
}

# => check_title|check_title_id # REQ_Y #
# => view_name   # REQ_Y #
# => align, default=center
# => read_only
# => data : ref to array with list rows, array consist of [id,title]
# => rows : area height in rows # NOT_REQ, default = 10
# => btns : ref to array with buttons for list operations, array consist of ref to hash with btn info
# => colspan : # REQ_N # XXX value for <td colspan=XXX> 
####################################
#	btns setup format:
#	  custom button #
#    => #title : button title 
#    => name : button name
#    => script : JS-script for button pressed
# 	  /custom button #
#	  system button #
#    => type : button type,variants
#			'remove' : remove selected items
#			'add' : create popup window for adding new items
#    => popup_url : for type=>'add'  : popup URL      
#    => popup_width : for type=>'add'  : popup window width ,default=500
#    => popup_height : for type=>'add'  : popup window height,default=400
#	  /system button #
sub cell_list{
	my $self = shift;	
	
	$self->{page}->cell_list(@_);	
}

sub cell_skip{
	my ($self,%h) = @_;
	
	$self->{page}->cell_skip(@_);
	#OLD	
	return;
	
	$self->{row}.= q{<td>&nbsp;</td>};
	$self->{col}++;
}


sub complete{
	my ($self,%h) = @_;
	
	$self->{page}->grid_push(@_);
	#OLD	
	return;
	
	$self->next_row() if $self->{col}>0;
	
	my $th={
		self=>$self
	};
	
	$self->{page}->{edit_out}.=	sw_web_parse_template(ref($self),'grid.tmpl',$th);
	
	
	# clean data #
	$self->_init($self->{cols});
}

sub grid_list {
  my ($class,%h) = @_;

  my $grid = new($class,page=>$h{page},cols=>$#{$h{info}->{cols}});

  foreach my $col (@{$h{info}->{cols}}) {
    next if $col->{no_select};
    $grid->cell_text(value=>string($col->{title_id}), view_name=>'aaa');

  }
  $grid->next_row();

  foreach my $col (@{$h{info}->{cols}}) {
    next if $col->{no_select};
    $grid->cell_skip();
  }
  $grid->next_row();

  foreach my $row (@{$h{info}->{data}}) {
    foreach my $col (@{$h{info}->{cols}}) {
      next if $col->{no_select};

      if ($col->{type} eq 'options') {
        $grid->cell_combo(value=>$row->[ $col->{loaded_index} ],
          view_name=>$col->{code},
          options=>$col->{data_options},
          options_plain=>1,
          read_only=>$col->{read_only}
        );
      }
      elsif ($col->{type} eq 'money') {
			  &sw_die("grid_list: The vendor_id attribute must be provided if type is money") unless ($col->{vendor_id});
        $grid->cell_input(value=>$row->[ $col->{loaded_index} ],
			view_name=>$col->{code}, max_length=>10, 
          prefix=>&sw_currency_prefix($col->{vendor_id}), postfix=>&sw_currency_postfix($col->{vendor_id}),
			read_only=>$col->{read_only}
        );
      }
      elsif ($col->{type} eq 'percent') {
        $grid->cell_input(value=>$row->[ $col->{loaded_index} ],
          view_name=>$col->{code}, max_length=>10, suffix=>'%',
          read_only=>$col->{read_only}
        );
      }
      else {
        $grid->cell_input(value=>$row->[ $col->{loaded_index} ],
          view_name=>$col->{code}, max_length=>20,
          read_only=>$col->{read_only}
        );
      }
    }
    $grid->next_row();
  }

  $grid->complete();
}


1;

__END__


