#!/usr/bin/perl

use HSPC::Console;
use HSPC::DataRequest;
use HSPC::WebDB;
use HSPC::MT::Billing::Constants;
use HSPC::MT::BM::HP::Constants;
use HSPC::MT::Billing::Subscription_base;
use HSPC::MT::BillingFactory;

my $req = HSPC::DataRequest->new(
	cols             => [qw(id plan_type create_order_id)],
	select_as_hashes => 1,
);

$req->set_where( 'status', [ EN_ACTIVE, EN_ON_HOLD, EN_PENDING, EN_GRACE, EN_EXPIRED, EN_TERMINATING ] );
$req->set_where( 'plan_type', [ BM_HP_TYPE_PLESK_DOMAIN, BM_HP_TYPE_PLESK_CLIENT ] );

HSPC::MT::Billing::Subscription_base->load_data( request => $req );

my $data = $req->data || [];
foreach my $row (@{$data}) {
	my $s_obj = HSPC::MT::Billing::Subscription_base->new();
	$s_obj->load(id=>$row->{id});
	my $o_obj;

	my $table = $row->{plan_type} == BM_HP_TYPE_PLESK_DOMAIN ? 'pleskdomain' : 'pleskclient';
	my $subscr = select_hash("select * from subscr_$table where subscr_id = ?", $row->{id});
	next unless $subscr;
	my $order;

        if($row->{create_order_id}) {
		$order = select_hash("select * from order_$table where ar_doc_id = ?",$row->{create_order_id});
		$o_obj = HSPC::MT::BillingFactory->ar_doc_find(
			id => $row->{create_order_id},
		);
        }

	my $add_data;
	if($row->{plan_type} == BM_HP_TYPE_PLESK_DOMAIN) {
		if($order) {
			$add_data->{exclusive_ip} = $order->{exclusive_ip};
			$add_data->{shared_ip} = $order->{shared_ip};
			$add_data->{forward_url} = $order->{forward_url};
		}
		$add_data->{pd_plesk_id} = $subscr->{pd_plesk_id};
		$add_data->{hn_id} = $subscr->{hn_id};
		$add_data->{hostname} = $subscr->{pd_hostname};
		$add_data->{ftp_login} = $subscr->{ftp_login};
		$add_data->{domain_password} = $subscr->{domain_password};
		$add_data->{phys_hosting_type} = $subscr->{phys_hosting_type};
		$add_data->{forward_url} = $subscr->{forward_url};
		$add_data->{force_ip} = $subscr->{force_ip};
	}
	else {
		if($order) {
			$add_data->{client_id} = $order->{client_id};
			$add_data->{plesk_login} = $order->{plesk_login};
			$add_data->{hw_id} = $order->{hw_id};
		}
		$add_data->{client_id} = $subscr->{client_id};
		$add_data->{hn_id} = $subscr->{hw_id};
		$add_data->{client_login} = $subscr->{client_login};
		$add_data->{pname_postfix} = $subscr->{pname_postfix};
	}
	$s_obj->add_params_data($add_data);
	$s_obj->save;
	if($o_obj) {
		$o_obj->add_params($s_obj->add_params);
		$o_obj->save;
	}
}
